@@ -33,7 +33,7 @@ net_init(void) {
     WSADATA wsa;
     int res = WSAStartup(MAKEWORD(2, 2), &wsa) < 0;
     if (res < 0) {
-        LOGC("WSAStartup failed with error %d", res);
+        LOGE("WSAStartup failed with error %d", res);
         return false;
     }
 #endif
@@ -83,6 +83,7 @@ unwrap(sc_socket socket) {
 #endif
 }
 
+#ifndef HAVE_SOCK_CLOEXEC // avoid unused-function warning
 static inline bool
 sc_raw_socket_close(sc_raw_socket raw_sock) {
 #ifndef _WIN32
@@ -91,6 +92,7 @@ sc_raw_socket_close(sc_raw_socket raw_sock) {
     return !closesocket(raw_sock);
 #endif
 }
+#endif
 
 #ifndef HAVE_SOCK_CLOEXEC
 // If SOCK_CLOEXEC does not exist, the flag must be set manually once the
@@ -115,14 +117,7 @@ set_cloexec_flag(sc_raw_socket raw_sock) {
 static void
 net_perror(const char *s) {
 #ifdef _WIN32
-    int error = WSAGetLastError();
-    char *wsa_message;
-    FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
-            NULL, error, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
-            (char *) &wsa_message, 0, NULL);
-    // no explicit '\n', wsa_message already contains a trailing '\n'
-    fprintf(stderr, "%s: [%d] %s", s, error, wsa_message);
-    LocalFree(wsa_message);
+    sc_log_windows_error(s, WSAGetLastError());
 #else
     perror(s);
 #endif
