@@ -8,7 +8,7 @@ static void test_buffer_write16be(void) {
     uint16_t val = 0xABCD;
     uint8_t buf[2];
 
-    buffer_write16be(buf, val);
+    sc_write16be(buf, val);
 
     assert(buf[0] == 0xAB);
     assert(buf[1] == 0xCD);
@@ -18,7 +18,7 @@ static void test_buffer_write32be(void) {
     uint32_t val = 0xABCD1234;
     uint8_t buf[4];
 
-    buffer_write32be(buf, val);
+    sc_write32be(buf, val);
 
     assert(buf[0] == 0xAB);
     assert(buf[1] == 0xCD);
@@ -30,7 +30,7 @@ static void test_buffer_write64be(void) {
     uint64_t val = 0xABCD1234567890EF;
     uint8_t buf[8];
 
-    buffer_write64be(buf, val);
+    sc_write64be(buf, val);
 
     assert(buf[0] == 0xAB);
     assert(buf[1] == 0xCD);
@@ -45,7 +45,7 @@ static void test_buffer_write64be(void) {
 static void test_buffer_read16be(void) {
     uint8_t buf[2] = {0xAB, 0xCD};
 
-    uint16_t val = buffer_read16be(buf);
+    uint16_t val = sc_read16be(buf);
 
     assert(val == 0xABCD);
 }
@@ -53,7 +53,7 @@ static void test_buffer_read16be(void) {
 static void test_buffer_read32be(void) {
     uint8_t buf[4] = {0xAB, 0xCD, 0x12, 0x34};
 
-    uint32_t val = buffer_read32be(buf);
+    uint32_t val = sc_read32be(buf);
 
     assert(val == 0xABCD1234);
 }
@@ -62,7 +62,7 @@ static void test_buffer_read64be(void) {
     uint8_t buf[8] = {0xAB, 0xCD, 0x12, 0x34,
                       0x56, 0x78, 0x90, 0xEF};
 
-    uint64_t val = buffer_read64be(buf);
+    uint64_t val = sc_read64be(buf);
 
     assert(val == 0xABCD1234567890EF);
 }
